<?php

namespace Drupal\cmlmerchant\Service;

use SimpleXMLElement;
use Drupal\taxonomy\Entity\Term;
use Drupal\image\Entity\ImageStyle;
use Drupal\commerce_product\Entity\Product;

/**
 * Class YmlService.
 */
class YmlService implements YmlServiceInterface {

  /**
   * SimpleXMLElement.
   *
   * @var \SimpleXMLElement
   */
  protected $xml;

  /**
   * Constructs a new YmlService object.
   */
  public function __construct() {
    $this->host = '';
    $this->xml = '';
    $this->shop = FALSE;
    $this->categories = [];
    $this->configUrl = FALSE;
    $this->config = \Drupal::config('cmlmerchant.settings');
    $this->getHost();
  }

  /**
   * {@inheritdoc}
   */
  public function debug() {
    $this->yandex();
    dsm($this->categories);
  }

  /**
   * {@inheritdoc}
   */
  public function yandex() {
    $this->xml = new SimpleXMLElement('<?xml version="1.0" encoding="utf-8"?><yml_catalog/>');
    $this->prepareYandexXml();
    $this->prepareData();
    $this->fillDataToYandexXml();
  }

  /**
   * {@inheritdoc}
   */
  public function google() {
    $this->xml = new SimpleXMLElement('<?xml version="1.0" encoding="utf-8"?><rss/>');
    $this->prepareGoogleXml();
    $this->prepareData();
    $this->fillDataToGoogleXml();
  }

  /**
   * Заполнение yandex xml общими данными.
   */
  private function prepareYandexXml() {
    // Текущая дата.
    $date = \Drupal::service('date.formatter')->format(time(), 'custom', 'Y-m-d\TH:i:s');
    $this->xml->addAttribute('date', $date);
    // Информация о магазине.
    $this->shop = $this->xml->addChild('shop');
    $this->shop->addChild('name', $this->config->get('shop_name'));
    $this->shop->addChild('company', $this->config->get('company_name'));
    $this->shop->addChild('url', $this->host);
    $this->shop->addChild('platform', 'Drupal');
    // Информация об используемых валютах.
    $this->fillCurrencyInfo();
  }

  /**
   * Заполнение google xml общими данными.
   */
  private function prepareGoogleXml() {
    $ns = ['g' => 'http://base.google.com/ns/1.0'];
    foreach ($ns as $prefix => $name) {
      $this->xml->registerXPathNamespace($prefix, $name);
    }
    $ns = (object) $ns;
    $this->xml->addAttribute('version', '2.0');
    $this->xml->addAttribute('xmlns:xmlns:g', $ns->g);

    $this->shop = $this->xml->addChild('channel');
    $this->shop->addChild('title', $this->config->get('company_name'));
    $this->shop->addChild('link', $this->host);
  }

  /**
   * Заполнение xml данными о валютах.
   */
  private function fillCurrencyInfo() {
    $currencies = $this->shop->addChild('currencies');
    $rurCur = $currencies->addChild('currency');
    $rurCur->addAttribute('id', 'RUB');
    $rurCur->addAttribute('rate', '1');
  }

  /**
   * Подготовка данных для подстановки в фид.
   */
  private function prepareData() {
    $this->getCategories();
    $this->getProducts();
  }

  /**
   * Заполнение xml фида данными о товарах.
   */
  private function fillDataToYandexXml() {
    $categories = $this->shop->addChild('categories');
    $offers = $this->shop->addChild('offers');
    foreach ($this->categories as $categoryId => $categoryInfo) {
      if (!empty($categoryInfo['products'])) {
        $this->setXmlCategory($categories, $categoryInfo);
        $this->setXmlProduct($offers, $categoryInfo);
      }
    }
    $str = $this->xml->asXML();
    $path = DRUPAL_ROOT . '/sites/default/files/YML';
    if (!file_exists($path)) {
      mkdir($path, 0775);
    }
    file_put_contents($path . '/cmlmerchant_yandex.xml', $str);
  }

  /**
   * Заполнение xml фида данными о товарах.
   */
  private function fillDataToGoogleXml() {
    foreach ($this->categories as $categoryId => $categoryInfo) {
      if (empty($categoryInfo['products']) || empty($categoryInfo['google_id'])) {
        continue;
      }
      foreach ($categoryInfo['products'] as $key => $productInfo) {
        foreach ($productInfo['variations'] as $variationInfo) {
          $item = $this->shop->addChild('item');
          $item->addChild("g:g:id", $productInfo['pid']);
          $item->addChild('title', $productInfo['title']);
          $item->addChild('g:g:description', $this->processDescription($productInfo['description']));
          $item->addChild('link', $productInfo['url']);
          $item->addChild('g:g:image_link', $productInfo['picture']);
          $item->addChild('g:g:availability', 'in stock');
          $item->addChild('g:g:price', $variationInfo['price'] . ' RUB');
          $item->addChild('g:g:google_product_category', $categoryInfo['google_id']);
        }
      }
    }
    $str = $this->xml->asXML();
    $path = DRUPAL_ROOT . '/sites/default/files/YML';
    if (!file_exists($path)) {
      mkdir($path, 0775);
    }
    file_put_contents($path . '/cmlmerchant_google.xml', $str);
  }

  /**
   * Запись категории в xml фид.
   */
  private function setXmlCategory(&$categories, $categoryInfo) {
    $termYml = $categories->addChild('category', $categoryInfo['name']);
    $termYml->addAttribute('id', $categoryInfo['id']);
    if (!empty($categoryInfo['parentId'])) {
      $termYml->addAttribute('parentId', $categoryInfo['parentId']);
    }
  }

  /**
   * Запись товара в xml фид.
   */
  private function setXmlProduct(&$offers, $categoryInfo) {
    foreach ($categoryInfo['products'] as $productInfo) {
      $this->setXmlProductVariation($offers, $productInfo);
    }
  }

  /**
   * Запись вариаций товара в xml фид.
   */
  private function setXmlProductVariation(&$offers, $productInfo) {
    foreach ($productInfo['variations'] as $variationInfo) {
      if (!empty($productInfo['picture'])) {
        $offer = $offers->addChild('offer');
        $offer->addAttribute('id', $productInfo['pid']);

        $offer->addChild('name', $productInfo['title']);
        $offer->addChild('url', $productInfo['url']);
        $offer->addChild('price', $variationInfo['price']);
        if (!empty($variationInfo['oldprice'])) {
          $offer->addChild('oldprice', $variationInfo['oldprice']);
        }
        $offer->addChild('currencyId', 'RUB');
        $offer->addChild('categoryId', $productInfo['categoryId']);
        if (!empty($productInfo['vendorCode'])) {
          $offer->addChild('vendorCode', $productInfo['vendorCode']);
        }
        foreach ($productInfo['picture'] as $img) {
          $offer->addChild('picture', $img);
        }
        // $offer->addChild('delivery', TRUE);
        if (!empty($productInfo['description'])) {
          $offer->addChild('description', $this->processDescription($productInfo['description']));
        }
        // $offer->addChild('sales_notes', 'Необходима предоплата.');
      }
    }
  }

  /**
   * Загрузка категорий.
   */
  private function getCategories($parent = 0, $child = FALSE) {
    $termsStorage = \Drupal::entityTypeManager()->getStorage('taxonomy_term');
    $terms = $termsStorage->loadTree('catalog', $parent, 1, TRUE);
    if (count($terms)) {
      foreach ($terms as $term) {
        $tid = $term->id();
        $category = [
          'name' => htmlspecialchars($term->getName()),
          'products' => [],
          'google_id' => $term->field_catalog_google_id->value,
          'id' => $tid,
        ];
        if ($child) {
          $category['parentId'] = $parent;
        }
        $this->categories[$tid] = $category;
        $this->getCategories($tid, TRUE);
      }
    }
  }

  /**
   * Загрузка товаров.
   */
  private function getProducts() {
    $categories = array_keys($this->categories);
    if (empty($categories)) {
      return;
    }
    $pids = \Drupal::entityQuery('commerce_product')
      ->condition('field_catalog', $categories, 'IN')
      ->condition('status', 1)
      ->execute();
    if (empty($pids)) {
      return;
    }
    $this->checkProducts($pids);
  }

  /**
   * Проверка товаров на наличие вариаций.
   */
  private function checkProducts($pids) {
    $products = Product::loadMultiple($pids);
    foreach ($products as $product) {
      if (!$product->hasVariations()) {
        continue;
      }
      $categoryId = $product->field_catalog->target_id;
      $item = [
        'pid' => $product->id(),
        'title' => $this->getProductTitle($product),
        'description' => $product->body->value ? htmlspecialchars($product->body->value) : '',
        'vendorCode' => $product->field_article->value ?? '',
        'categoryId' => $categoryId,
        'variations' => [],
        'picture' => $this->getProductPicture($product),
        'url' => $this->fixUrl($product->toUrl('canonical', ['absolute' => TRUE])->toString()),
      ];
      $this->checkVariations($item, $product->getVariations());
      if (!empty($item['variations'])) {
        $this->categories[$categoryId]['products'][$product->id()] = $item;
      }
    }
  }

  /**
   * Определение названия товара.
   */
  private function getProductTitle($product) {
    $siteProductnameField = $product->field_title;
    if (!$siteProductnameField->isEmpty()) {
      $productTitle = $siteProductnameField->value;
    }
    $title = empty($productTitle) ? $product->getTitle() : $productTitle;
    return htmlspecialchars($title);
  }

  /**
   * Заполнение товара информацией о вариации.
   */
  private function checkVariations(&$item, $variations) {
    foreach ($variations as $variation) {
      $price = (float) $variation->price->number;
      $stock = $variation->field_stock->value;
      if (!$variation->isActive() || $stock === 0 || !$price) {
        continue;
      }
      $variationInfo = [
        'price' => $price,
        'oldprice' => (float) $variation->field_oldprice->value > $price ? $variation->field_oldprice->value : '',
      ];
      $item['variations'][] = $variationInfo;
      break;
    }
  }

  /**
   * Получение изображения товара.
   */
  private function getProductPicture($product) {
    $images = [];
    if (is_object($image = $product->field_image->entity)) {
      $imageUri = $image->getFileUri();
      $url = file_create_url($imageUri);
      $url = $this->fixUrl($url);
      $images[] = $url;
    }
    if (!empty($product->field_gallery)) {
      foreach ($product->field_gallery as $image) {
        $imageUri = $image->entity->getFileUri();
        $url = file_create_url($imageUri);
        $url = $this->fixUrl($url);
        if (count($images) < 10) {
          $images[] = $url;
        }
      }
    }
    return $images;
  }

  /**
   * Получение изображения товара (было).
   */
  private function getProductPictureOld($product) {
    $field = $product->field_gallery;
    if (!$field->count()) {
      $field = $product->field_image;
      if ($field->isEmpty()) {
        return '';
      }
    }
    if (is_object($image = $field->entity)) {
      $imageUri = $image->getFileUri();
      // $url = ImageStyle::load('product_gallery_big')->buildUrl($imageUri);
      // $url = explode('?itok', $url)[0];
      $url = file_create_url($imageUri);
      $url = $this->fixUrl($url);
      return $url;
    }
    return '';
  }

  /**
   * Получение хоста.
   */
  private function getHost() {
    $this->configUrl = $this->config->get('site_url');
    $this->host = $this->fixUrl(\Drupal::request()->getSchemeAndHttpHost() . '/');
  }

  /**
   * В случаях, когда url начинается на http://default, заменяем его на url из конфига.
   */
  private function fixUrl($url) {
    if (!empty($this->configUrl)) {
      $url = str_replace('http://default', $this->configUrl, $url);
    }
    return $url;
  }

  /**
   * Оформляем описание.
   */
  private function processDescription($description) {
    return '<![CDATA[' . $description . ']]>';
  }

}
